#! /usr/bin/env python3
"""
Kubernetes VirtualMachine Monitor

This script monitors a Kubernetes cluster for newly deployed VirtualMachines (VMs),
which are custom resources managed by KubeVirt and executes startup script if exist.

Events on that vm_state_handler.py script is called:
- apply
- delete
- start
- stop
"""

import os
import logging
import time
import sys
import socket

from vm_state_handler import main as run_second_script
from isNodeReady import get_is_node_ready, get_kube_system_pods_ready
from kubernetes import client, config, watch

cbox_file = "/config/k8s-single-machine"
central_controller = "/config/central-box"
FLAG_FILES_DIR = "/config/vnf/"
MAX_RETRIES = 10
WAIT_SECONDS = 60

logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s [%(levelname)s] - %(message)s',
    handlers=[
        logging.StreamHandler()
    ]
)

def is_flag_file_exists(vm_key):
    flag_file_path = f"{FLAG_FILES_DIR}run_{vm_key}"
    return os.path.isfile(flag_file_path)

def create_flag_file(vm_key):
    namespace, vm_name = vm_key
    flag_file_path = os.path.join('/config/vnf/', f'run_{namespace}_{vm_name}')
    open(flag_file_path, 'w').close()  # Create a flag file

def delete_flag_file(vm_key):
    namespace, vm_name = vm_key
    flag_file_path = os.path.join('/config/vnf/', f'run_{namespace}_{vm_name}')
    if os.path.exists(flag_file_path):
        os.remove(flag_file_path)  # Delete flag file if it exists

def get_flag_file_path(vm_key):
    namespace, vm_name = vm_key
    return os.path.join('/config/vnf/', f'run_{namespace}_{vm_name}')

def flag_file_exists(vm_key):
    return os.path.exists(get_flag_file_path(vm_key))

def dict_compare(d1, d2, excluded_keys):
    for k in set(d1.keys()).union(d2.keys()):
        if k not in excluded_keys:
            if d1.get(k) != d2.get(k):
                return False
    return True

def watch_vm_creation():
    """
    This function is used to watch VM status and call
    vm_state_handler.py script to apply boot script on
    specific conditions of VM: start, stop, delete, apply.
    """
    # Define the Custom Resource details
    group = 'kubevirt.io'
    version = 'v1'
    namespace = 'kubevirt'
    plural = 'virtualmachines'

    # Load Kubernetes configuration
    config.load_kube_config()
    # Initialize the API client for Kubevirt
    custom_api = client.CustomObjectsApi()
    w = watch.Watch()

    logging.info("Started monitoring for new VirtualMachines...")

    try:
        for event in w.stream(custom_api.list_cluster_custom_object, group, version, plural):
            if event:
                if 'object' in event and 'status' in event['object']:
                    vm = event['object']
                    vm_name = vm['metadata']['name']
                    namespace = vm['metadata']['namespace']
                    vm_key = (namespace, vm_name) # unique key for this VM
                    status = event['type']
                    running = vm['spec']['running'] # if VM is in 'stopping' state

                    if status == 'DELETED':
                        logging.info(f"VirtualMachine '{vm_name}' in '{namespace}' namespace is with status '{status}'.")
                        delete_flag_file(vm_key)
                    elif (status in ['ADDED', 'MODIFIED'] and
                        event['object']['status'].get('printableStatus') == 'Running' and
                        str(event['object']['status'].get('ready')) == "True" and
                        'deletionTimestamp' not in vm["metadata"] and running):
                        if not flag_file_exists(vm_key):
                            try:
                                logging.info(f"VirtualMachine '{vm_name}' in '{namespace}' namespace is with status '{status}'.")
                                create_flag_file(vm_key)
                                # Call /scripts/vm_state_handler.py
                                run_second_script(namespace, vm_name)
                            except Exception as e:
                                logging.error(f"An error occurred while running the second script: {e}")
                                if flag_file_exists(vm_key):
                                    delete_flag_file(vm_key)

                    if not 'vnf-onboot-script-once' in vm['metadata'].get('annotations', {}):
                        delete_flag_file(vm_key)
    except Exception as e:
        logging.exception(f"An error occurred into watching for VirtualMachine statuses: {e}.")
        sys.exit(1)

def main():
    if not os.path.exists(cbox_file) and not os.path.exists(central_controller):
        logging.info("Only available for CBOX or Central Controler")
        exit(0)

    retry_count = 0
    controller_hostname = socket.gethostname()

    while retry_count < MAX_RETRIES:
        try:
            # Check that controller node and all system PODs are up & running, then call watch logic
            if get_is_node_ready(controller_hostname) and get_kube_system_pods_ready(controller_hostname):
                watch_vm_creation()
            else:
                retry_count += 1
                if retry_count < MAX_RETRIES:
                    logging.info(f"Retrying in {retry_count * 10} seconds... Controller '{controller_hostname}' is not in 'Running' state.")
                    time.sleep(retry_count * 10)  # Wait for 10, 20, 30, 40 seconds etc. before retrying
                else:
                    logging.error(f"An error occurred: {e}. Exiting after {MAX_RETRIES} attempts.")
                    sys.exit(1)
        except Exception as e:
            logging.exception(f"An error occurred: {e}.")
            sys.exit(1)

if __name__ == '__main__':
    main()
