#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import sys
import syslog
from ipaddress import IPv4Network, ip_address
from enum import Enum
from tveapi.common import utils

from sysrepo_interfaces import sysrepo_interfaces3 as api
from sysrepo_wg_concentrator import sysrepo_wg_concentrator as wg_api

class ValidateErrors(Enum):
    OK = 0
    NOT_FOUND = -1
    IP_DUP_FOUND = -2
    METRIC_DUP_FOUND = -3
    NO_MGMNT = -4
    NOVALUE_IN_SYSREPO = -5
    
def getErrorAsText(error, optional_text=""):
    errorText = None

    if error == ValidateErrors.OK:
        errorText = ""
    elif error == ValidateErrors.NOT_FOUND:
        errorText = "Validation Error: Interface '%s' does not exist!" % optional_text
    elif error == ValidateErrors.IP_DUP_FOUND:
        errorText = "Validation Error: Another interface - '%s' with the same IP found!" % optional_text
    elif error == ValidateErrors.METRIC_DUP_FOUND:
        errorText = "Validation Error: Another interface - '%s' has the same type bootproto, metric [and set gateway, if present]!" % optional_text
    elif error == ValidateErrors.NO_MGMNT:
        errorText = "Validation Error: At least one management should be present!"
    elif error == ValidateErrors.NOVALUE_IN_SYSREPO:
        errorText = "Validation Error: Could not retrieve interface data!"

    return errorText

def validate_interface_by_type(concentrator_api, ifname, iftype):
    sri = api.SysrepoInterfaces(app_name="set wireguard interfaces")

    try:
        if ifname in concentrator_api.get_wireguard_concentrator_interfaces()['wg-concentrator']:
            return
    except:
        sri.set_interface(ifname)
        concentrator_api.set_wireguard_concentrator_param(ifname, 'type', iftype)

def verify_if_intf_with_name_already_exists_by_type(name):
    interfaces = api.SysrepoInterfaces(app_name="set interfaces")
    return interfaces.get_interface_param(name, 'type')

def del_wireguard_conc_int(name):
    concentrator_api = wg_api.SysrepoWireGuardConcentrator(app_name="wireguard concentrator")
    concentrator_api.del_wireguard_concentrator(name)
    concentrator_api.commit()

def del_wireguard_peer(intf_name, c_choice, c_param):
    if not intf_name and not c_param:
        print("Error applying WireGuard concentrator configuration.")
        return

    code, stdout, stderr = utils.execute_shell_command(
            f"/usr/bin/sudo python3 /scripts/set_wireguard_concentrator.py delete_client {intf_name} {c_choice} {c_param}")
    print(stdout)

def set_wireguard_conc_int(name, param, value):
    if not param or not value:
        print("Error applying WireGuard concentrator configuration.")
        return

    concentrator_api = wg_api.SysrepoWireGuardConcentrator(app_name="wireguard concentrator")

    int_type = verify_if_intf_with_name_already_exists_by_type(name)
    if int_type:
        print(f"Interfacewith name {name} is already in use by {int_type}! Please choose another one!")
        return

    validate_interface_by_type(concentrator_api, name, 'wireguard-concentrator')

    if param == "concentrator-port":
        if int(value) in range(1, 65536):
            concentrator_api.set_wireguard_concentrator_param(name, param, int(value))
        else:
            error_message = "Wrong port number is given. Port number must be in the range: 1 to 65535."
            syslog.syslog(syslog.LOG_ERR, error_message)
            print(error_message)
            return
    else:
        concentrator_api.set_wireguard_concentrator_param(name, param, value)

    if param == 'enable' and value == 'true':
        print("Note: Wireguard Concentrator will act as a DNS server.")

    concentrator_api.commit()
    
def set_concentrator_params(type, name, param, value):
    if not type or not param or not value:
        print("Error applying WireGuard concentrator configuration.")
        return
    
    concentrator_api = wg_api.SysrepoWireGuardConcentrator(app_name="wireguard concentrator")
    concentrator_api.set_wireguard_concentrator_param(name, param, value)
    concentrator_api.commit()

def set_external_peers(name, endpoint, param, value):
    if not endpoint or not param or not value:
        print("Error applying WireGuard concentrator configuration.")
        return
        
    concentrator_api = wg_api.SysrepoWireGuardConcentrator(app_name="wireguard concentrator")
    concentrator_api.set_wireguard_concentrator_external_peers(name, endpoint, param, value)
    concentrator_api.commit()

def del_external_peers(name, endpoint):
    if not endpoint:
        print("Error applying WireGuard concentrator configuration.")
        return
    
    concentrator_api = wg_api.SysrepoWireGuardConcentrator(app_name="wireguard concentrator")
    concentrator_api.del_wireguard_concentrator_external_peers(name, endpoint)
    concentrator_api.commit()
    
def set_static_clients(name, hostname, param, value):
    if not hostname or not param or not value:
        print("Error applying WireGuard concentrator configuration.")
        return
    
    code, stdout, stderr = utils.execute_shell_command(
            f"/usr/bin/sudo python3 /scripts/set_wireguard_concentrator.py static {name} {hostname} {value}")
    print(stdout)
    
def del_static_clients(name, hostname):
    if not hostname:
        print("Error applying WireGuard concentrator configuration.")
        return
    
    code, stdout, stderr = utils.execute_shell_command(
            f"/usr/bin/sudo python3 /scripts/set_wireguard_concentrator.py static {name} {hostname} delete")
    
def set_access_list(name, acl_type, s_ip, d_ip, delete=False):
    concentrator_api = wg_api.SysrepoWireGuardConcentrator(app_name="wireguard concentrator")

    if not (validate_ip_cidr(s_ip) and validate_ip_cidr(d_ip)):
        print("Please add a valid IP address with CIDR notation.")
        return

    value = f"src {s_ip} dst {d_ip}"

    if not delete:
        acl_id = concentrator_api.get_wireguard_concentrator_next_acl_id(name, acl_type)
        concentrator_api.set_wireguard_concentrator_acl(name, acl_type, acl_id, value)
        concentrator_api.commit()
    else:
        #search for specific value and unset the id
        acl_id = concentrator_api.find_wireguard_concentrator_acl_rule(name, acl_type, value)
        if acl_id:
            concentrator_api.unset_wireguard_concentrator_acl(name, acl_type, acl_id)
            concentrator_api.commit()
        else:
            print(f"ACL {value} does not exist!")
            return

def validate_ip_cidr(ip_cidr):
    """
    Validate the provided IP address with CIDR notation.

    Parameters:
    - ip_cidr (str): The IP address in CIDR format (e.g., '192.168.1.0/24').

    Returns:
    - bool: True if valid, False if not valid.
    - str: A message indicating validity or error.
    """
    try:
        # Create an IPv4Network object to validate the CIDR
        network = IPv4Network(ip_cidr)
        return True
    except ValueError as e:
        return False


if __name__ == "__main__":
    argc = len(sys.argv)

    if 3 > argc:
        syslog.syslog(syslog.LOG_ERR, "No enough arguments. Exit!")
        sys.exit(1)
        
    if sys.argv[1] == "set_wireguard_conc_int":
        if sys.argv[3] == "delete":
            del_wireguard_conc_int(sys.argv[2])
        else:
            set_wireguard_conc_int(sys.argv[2], sys.argv[3], sys.argv[4])
    elif sys.argv[1] == "set_ip_ranges":
        set_concentrator_params(sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4])
    elif sys.argv[1] == "set_domain_name":
        set_concentrator_params(sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4])
    elif sys.argv[1] == "set_static_routes":
        set_concentrator_params(sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4])
    elif sys.argv[1] == "set_external_peers":
        if sys.argv[4] == "delete":
            del_external_peers(sys.argv[2], sys.argv[3])
        else:
            set_external_peers(sys.argv[2], sys.argv[3], sys.argv[4], sys.argv[5])
    elif sys.argv[1] == "set_static_clients":
        if sys.argv[4] == 'delete':
            del_static_clients(sys.argv[2], sys.argv[3])
        else:
            set_static_clients(sys.argv[2], sys.argv[3], sys.argv[4], sys.argv[5])
    elif sys.argv[1] == "set_access_list":
        if argc == 7:
            set_access_list(sys.argv[2], sys.argv[3], sys.argv[4], sys.argv[5], delete=True)
        else:
            set_access_list(sys.argv[2], sys.argv[3], sys.argv[4], sys.argv[5])
    elif sys.argv[1] == "delete-client":
        del_wireguard_peer(sys.argv[2], sys.argv[3], sys.argv[4])
    else:
        syslog.syslog(syslog.LOG_ERR, "Unknown argument \"%s\"" % sys.argv[1])
        sys.exit(1)

    sys.exit(0)
