#!/bin/bash
#/ Usage: sudo chroot /var/tmp/helm_bash_jail/ bin/bash
#/
#/ This script operates in bash jail and serves to users 
#/ interactive Helm console.

HELM="binaries/helm"
# HELM command arguments
# 'templates/' folder is mounted to '/kubernetes/helm/' host directory
HELM_CUSTOM_FLAGS="--kubeconfig config/config \
                --registry-config templates/.config/helm/registry/config.json \
                --repository-cache templates/.cache/helm/repository \
                --repository-config templates/.config/helm/repositories.yaml"

alias help='helm help'
alias history='helm history'
alias tar='binaries/tar'
alias grep='binaries/grep'
alias head='binaries/head'
alias rm='binaries/rm'

# Do not allow bash completion
bind 'set disable-completion on'

# Print helm version on startup
export PS1="helm\$ "
echo -e "Welcome to Helm interactive shell.\n$($HELM version --template='Version: {{.Version}}')"

# Define the list of commands and keywords to exclude
excluded_commands=(
    "bash"       "bg"         "bind"       "break"     "builtin"
    "caller"     "case"       "cd"         "command"   "compgen"
    "complete"   "compopt"    "continue"   "coproc"    "declare"
    "dirs"       "disown"     "elif"       "else"      "enable"
    "esac"       "exec"       "false"      "fc"        "fg"
    "getopts"    "hash"       "jobs"       "kill"      "let"
    "local"      "logout"     "mapfile"    "popd"      "printf"
    "pushd"      "pwd"        "rbash"      "read"      "readarray"
    "readonly"   "select"     "set"        "shift"     "shopt"
    "source"     "suspend"    "test"       "time"      "times"
    "true"       "type"       "typeset"    "ulimit"    "umask"
    "unalias"    "unset"      "until"      "wait"      "while"
)

# This function restricts bash commands (especially kill command)
function restrict_command() {
    for cmd in ${excluded_commands[@]}; do
        if [[ " ${excluded_commands[*]} " =~ " $cmd " ]]; then
            alias $cmd=''
        fi
    done
}

# Do nothing on missing commands
function command_not_found_handle() {
    :
}

# This function is called by bash trap
function preexec_invoke_exec () {
    CMD_COMMAND=$1
    [ -n "$COMP_LINE" ] && return  # do nothing if completing
    [ "$CMD_COMMAND" = "$PROMPT_COMMAND" ] && return # don't cause a preexec for $PROMPT_COMMAND

    if [[ "$CMD_COMMAND" != *"helm"* ]]; then
        command="$HELM $HELM_CUSTOM_FLAGS $CMD_COMMAND"
    else
        command="binaries/$CMD_COMMAND $HELM_CUSTOM_FLAGS"
    fi

    if [[ $(echo "${CMD_COMMAND}" | grep -wo "uninstall") ]]; then
        release=$(${command} --dry-run | grep -oP '"\K[^"]+' | head -n 1)
        if [[ ${release} != "" ]]; then
            rm -f templates/${release}.tar
        fi
    fi

    eval "${command}"

    #if [[ "${CMD_COMMAND}" == *"install "* || "${CMD_COMMAND}" == *"upgrade "* ]]; then
    if [[ $(echo "${CMD_COMMAND}" | grep -wo "install") || $(echo "${CMD_COMMAND}" | grep -wo "upgrade") ]]; then
        release=$(${command} --dry-run -o json | grep -oP '"name":"\K[^"]+' | head -n 1)
        if [[ ${release} != "" ]]; then
            $HELM status ${release} --kubeconfig config/config --show-resources -o json > ${release}_helm_status.yaml
            $HELM get all ${release} --kubeconfig config/config > ${release}_helm_resources.yaml
            tar cf templates/${release}.tar ${release}_helm_status.yaml ${release}_helm_resources.yaml
        fi
    fi
}

# Restrict bash commands
restrict_command

# Make Bash trap
trap '
if [[ "$BASH_COMMAND" == "exit" || "$BASH_COMMAND" == "quit" || "$BASH_COMMAND" == *"binaries/rm"* || "$BASH_COMMAND" == *"binaries/tar"* ]]; then
    exit 0
fi
preexec_invoke_exec "$BASH_COMMAND"
' DEBUG
