#!/usr/bin/env python3

import sys
import syslog
import yaml
import vppapi_if.vppapi_if as vppif
from sysrepo_interfaces import sysrepo_interfaces3 as sri_api
from sysrepo_wg_concentrator import sysrepo_wg_concentrator as wg_api
import edgility_db.edgility_db as db


def verify_if_wireguard_concentrator(concentrator_api, intf_name):
    wg_type = concentrator_api.get_wireguard_concentrator_param(intf_name, 'type')
    if wg_type and wg_type == 'wireguard-concentrator':
        return True
    return None

def get_config_wireguard_interfaces():
    sri = sri_api.SysrepoInterfaces(app_name='show-wireguard')
    
    config = sri.get_interfaces()
    wg_interfaces = [v for k, v in config.items() if k.startswith('wg')]

    formatted_config = {
        'ucpe:config': {
            'interfaces:interface': wg_interfaces
        }
    }

    print(yaml.dump(formatted_config, default_flow_style=False))

def get_config_wireguard_concentrator():
    wgi = wg_api.SysrepoWireGuardConcentrator(app_name='show-wireguard')
    
    config = wgi.get_wireguard_concentrator_interfaces()
    wg_interfaces = [v for k, v in config.items() if k.startswith('wg')]

    formatted_config = {
        'ucpe:config': {
            'interfaces:interface': wg_interfaces
        }
    }

    print(yaml.dump(formatted_config, default_flow_style=False))
    
def get_config_wireguard_public_key():
    wgi = wg_api.SysrepoWireGuardConcentrator(app_name='show-wireguard')
    
    data = wgi.get_wireguard_interfaces_from_vpp()
    filtered_data = [{'interface': item['ifname'], 'public_key': item['public_key']} for item in data]

    formatted_data = {
        'ucpe:config': {
            'interfaces:interface': filtered_data
        }
    }

    print(yaml.dump(formatted_data, default_flow_style=False))

def get_wireguard_active_sessions():
    wgi = wg_api.SysrepoWireGuardConcentrator(app_name='show-wireguard')

    data = wgi.get_wireguard_peers_from_vpp()
    if data:
        print(yaml.dump(data, default_flow_style=False))
    else:
        print("Currently there are no active wireguard tunnel sessions.")

def get_wireguard_available_ips(intf):
    wgi = wg_api.SysrepoWireGuardConcentrator(app_name='show-wireguard')
    
    if not verify_if_wireguard_concentrator(wgi, intf):
        return
    
    available_ips = wgi.get_wireguard_available_ips(intf)
    if available_ips:
        if len(available_ips) <= 0:
            print("THere are no available IP addresses.")
        else:
            print(yaml.dump(available_ips, default_flow_style=False))
    else:
        print(f"Tunnel IP for interface {intf} has not been set!")

def get_wireguard_dynamic_peers(intf):
    wgi = wg_api.SysrepoWireGuardConcentrator(app_name='show-wireguard')
    
    if not verify_if_wireguard_concentrator(wgi, intf):
        print(f'Interface {intf} is not configured as Wireguard Concentrator.')
        return
    
    data = wgi.get_wireguard_dynamic_peers_from_db(intf)
    if data:
        print(yaml.dump(data, default_flow_style=False))
    else:
        print(f"There is no data for interface {intf}.")

def get_wireguard_static_peers(intf):
    wgi = wg_api.SysrepoWireGuardConcentrator(app_name='show-wireguard')
    
    if not verify_if_wireguard_concentrator(wgi, intf):
        print(f'Interface {intf} is not configured as Wireguard Concentrator.')
        return
    
    data = wgi.get_wireguard_static_peers_from_db(intf)
    if data:
        print(yaml.dump(data, default_flow_style=False))
    else:
        print(f"There is no data for interface {intf}.")


if __name__ == "__main__":
    argc = len(sys.argv)

    if 2 > argc:
        syslog.syslog(syslog.LOG_ERR, "No enough arguments. Exit!")
        sys.exit(1)

    if sys.argv[1] == "get_config_wireguard_interfaces":
        get_config_wireguard_interfaces()
    elif sys.argv[1] == "get_config_wireguard_concentrator":
        get_config_wireguard_concentrator()
    elif sys.argv[1] == "get_config_wireguard_public_key":
        get_config_wireguard_public_key()
    elif sys.argv[1] == "get_wireguard_active_sessions":
        get_wireguard_active_sessions()
    elif sys.argv[1] == "get_wireguard_available_ips":
        get_wireguard_available_ips(sys.argv[2])
    elif sys.argv[1] == "get_wireguard_dynamic_peers":
        get_wireguard_dynamic_peers(sys.argv[2])
    elif sys.argv[1] == "get_wireguard_static_peers":
        get_wireguard_static_peers(sys.argv[2])

    else:
        syslog.syslog(syslog.LOG_ERR, "Unknown argument \"%s\"" % sys.argv[1])
        sys.exit(1)

    sys.exit(0)
