#!/usr/bin/env python3

import requests
import json
import os
import sys
import logging
import logging.handlers
from requests.packages.urllib3.exceptions import InsecureRequestWarning
from sysrepo_api import sysrepo_api3 as api
from time import sleep

CONFIG_NAMESPACE = '/ucpe:config'

sysrepo_client = None
deleted_data = {'deleted': ''}
manager_server_headers = {'Content-Type': 'application/json', }
manager_server_url = 'https://{}:{}/config/helm'
ztp_port = 30636
log = None

def set_log(name, level):
    """
    Set log level.
    """
    global log
    log = logging.getLogger(name=name)
    log.setLevel(level=level)

    syslog_hnd = logging.handlers.SysLogHandler(address='/dev/log')
    formatter = logging.Formatter('%(name)s: %(levelname)s %(message)s')
    syslog_hnd.setFormatter(formatter)
    log.addHandler(syslog_hnd)

def sendHelmConfig(create, config):
    """
    sendHelmConfig trying in loop to send config of deployed Helm
    chart to Manager.
    """
    global ztp_port
    global sysrepo_client

    # Get machine serial number
    serial_number = None

    manager_server = None

    if sysrepo_client:
        del sysrepo_client
        sysrepo_client = None
    try:
        if sysrepo_client is None:
            sysrepo_client = api.SysrepoAPI(app_name='sendHelmStatus',log_level=log_level)
            sysrepo_req = CONFIG_NAMESPACE + '/system:system/config-server/server'
            manager_server = sysrepo_client.get_item(sysrepo_req)
    except:
        log.error('Unable to get data from sysrepo!')
        return

    try:
        serial_file = open('/config/.serial-number', 'r')
        serial_number = serial_file.read().strip()
        serial_file.close()
    except (OSError, IOError):
        log.error('Error opening file contains serial number!')
        return

    # Send data to server if allowed
    if serial_number and manager_server:
        # Create retry loop
        retry_loop = True
        retry_seconds = 5
        while retry_loop:
            manager_server_params = (('serial-number', serial_number),)
            try:
                log.debug(f'Try upload Helm status to {manager_server}:{ztp_port}')

                if create and not os.path.exists(config):
                    log.error('Error opening file contains Helm config!')
                    break

                if create:
                    with open(config, 'rb') as input:
                        response = requests.put(manager_server_url.format(manager_server, ztp_port),
                                                headers=manager_server_headers,
                                                params=manager_server_params, data=input.read(),
                                                verify=False, timeout=(5, 5))
                else:
                    filename = os.path.basename(config)
                    helm_dep_name = os.path.splitext(filename)[0]
                    deleted_data['deleted'] = helm_dep_name
                    response = requests.put(manager_server_url.format(manager_server, ztp_port),
                                            headers=manager_server_headers,
                                            params=manager_server_params, data=json.dumps(deleted_data),
                                            verify=False, timeout=(5, 5))

                if response.status_code == requests.codes.ok:
                    log.info(f'Upload Helm status to {manager_server}:{ztp_port} SUCCEEDED!')
                    retry_loop = False  # Exit from retry loop
                    break

                else:
                    log.info('Upload Helm status to {}:{} FAILED with {}!'.format(
                                manager_server, ztp_port, response.status_code))
                    if response.status_code == 500:
                        log.debug('Please, check serial number!')

                    log.debug(f'Retry in {retry_seconds} seconds...')
                    sleep(retry_seconds)

            except Exception as e:
                log.info(f'Upload Helm status to {manager_server}:{ztp_port} exception {e}')
                log.debug('Please, check resolving/connection to that address/IP')
                log.debug(f'Retry in {retry_seconds} seconds...')
                sleep(retry_seconds)

            if retry_seconds >= 30:
                retry_seconds = 30
            else:
                retry_seconds += 1

    return

if __name__ == '__main__':
    """Send Helm config to manager

    This script allows the controller to send deployed Helm template to
    the Manager.

    For DEBUG, user can call script with '--debug' option and get all
    logs with 'journalctl | grep sendHelmStatus'.

    """

    debugmode = False
    create = True
    log_level = logging.INFO

    for arg in sys.argv[1:]:
        if arg == '--debug':
            debugmode = True
            log_level = logging.DEBUG
            break
        if sys.argv[3] == 'IN_CREATE':
            create = True
            break
        if sys.argv[3] == 'IN_DELETE':
            create = False
            break

    helm_config = '/kubernetes/helm/' + sys.argv[2]

    set_log('sendHelmStatus', log_level)
    if not debugmode:
        requests.packages.urllib3.disable_warnings(InsecureRequestWarning)

    sendHelmConfig(create, helm_config)
